import 'package:flutter_slick/navigation/route_builder.dart';
import 'package:flutter_slick/navigation/route_path.dart';

import '../screens/not_found.dart';
import '../screens/entity.dart';
import '../screens/sub_entity.dart';
import '../utils/navigation.dart';

final RouteBuilder<EntityArguments> root = RouteBuilder(
  pattern: RegExp(r'^entity/(?<id>.*)/(.*)$'),
  path: (args, subPath) => 'entity/${args.id}/$subPath',
  parser: EntityArguments.parse,
  paths: [
    view,
    viewSub,
  ],
  build: (_, args, __) async => const NotFoundScreen(),
  rewrite: never,
);

final view = RouteBuilder(
  pattern: RegExp(r'^view$'),
  path: (args, _) => 'view',
  parser: RouteArguments.parse,
  build: (_, args, parent) async {
    if (parent == null) return const NotFoundScreen();
    return EntityScreen(parent: parent as RoutePath<EntityArguments>);
  },
  rewrite: never,
);

final rootSub = RouteBuilder(
  parser: EntityArguments.parse,
  pattern: RegExp(r'^sub/(?<id>.*)/(.*)$'),
  path: (args, subPath) => 'sub/${args.id}/$subPath',
  paths: [
    viewSub,
  ],
  build: (_, __, ___) async {
    return const NotFoundScreen();
  },
  rewrite: never,
);

final viewSub = RouteBuilder(
  pattern: RegExp(r'^view$'),
  path: (args, _) => 'view',
  parser: RouteArguments.parse,
  build: (_, args, parent) async {
    if (parent == null) return const NotFoundScreen();
    var entityParent = parent.parent;
    if (entityParent == null) return const NotFoundScreen();
    return SubEntityScreen(
      entityId: (entityParent.args as EntityArguments).id,
      id: (parent.args as EntityArguments).id,
    );
  },
  rewrite: never,
);
