import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class DecimalTextInputFormatter extends TextInputFormatter {
  RegExp? _exp;

  DecimalTextInputFormatter({
    int decimalRange = 3,
    bool activatedNegativeValues = true,
  }) : assert(
            decimalRange >= 0, 'DecimalTextInputFormatter declaretion error') {
    var dp = (decimalRange > 0) ? '([.][0-9]{0,$decimalRange}){0,1}' : '';
    var num = '[0-9]*$dp';
    if (activatedNegativeValues) {
      _exp = RegExp('^((((-){0,1})|((-){0,1}[0-9]$num))){0,1}\$');
    } else {
      _exp = RegExp('^($num){0,1}\$');
    }
  }

  @override
  TextEditingValue formatEditUpdate(
    TextEditingValue oldValue,
    TextEditingValue newValue,
  ) {
    if (_exp!.hasMatch(newValue.text)) {
      return newValue;
    }
    return oldValue;
  }
}

class CurrencyField extends StatelessWidget {
  final double? initialValue;
  final FormFieldValidator? validator;
  final FocusNode? focusNode;
  final TextEditingController? contorller;
  final bool autofocus;
  final bool enabled;
  final String labelText;
  final String currency;
  final void Function(double?) onChanged;

  const CurrencyField({
    super.key,
    this.initialValue,
    this.validator,
    this.focusNode,
    this.contorller,
    this.autofocus = false,
    this.enabled = true,
    required this.labelText,
    required this.currency,
    required this.onChanged,
  });

  @override
  Widget build(BuildContext context) {
    return TextFormField(
      initialValue: initialValue.toString(),
      validator: validator,
      keyboardType: const TextInputType.numberWithOptions(decimal: true),
      inputFormatters: [DecimalTextInputFormatter()],
      focusNode: focusNode,
      controller: contorller,
      autofocus: autofocus,
      decoration: InputDecoration(
        enabled: enabled,
        labelText: labelText,
        prefixText: currency,
      ),
      onChanged: (v) => double.tryParse(v),
    );
  }
}
