import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class NumberField extends StatelessWidget {
  final int? initialValue;
  final FormFieldValidator? validator;
  final FocusNode? focusNode;
  final TextEditingController? controller;
  final bool autofocus;
  final bool enabled;
  final String labelText;
  final void Function(int?) onChanged;

  const NumberField({
    super.key,
    this.initialValue,
    this.validator,
    this.focusNode,
    this.controller,
    this.autofocus = false,
    this.enabled = true,
    required this.labelText,
    required this.onChanged,
  });

  @override
  Widget build(BuildContext context) {
    return TextFormField(
      initialValue: initialValue.toString(),
      validator: validator,
      keyboardType: TextInputType.number,
      inputFormatters: [
        FilteringTextInputFormatter.digitsOnly,
      ],
      focusNode: focusNode,
      controller: controller,
      autofocus: autofocus,
      enabled: enabled,
      onChanged: (v) => onChanged(int.tryParse(v)),
      decoration: InputDecoration(
        enabled: enabled,
        labelText: labelText,
      ),
    );
  }
}
