import 'package:flutter/material.dart';

class PasswordField extends StatefulWidget {
  final String? initialValue;
  final FormFieldValidator<String>? validator;
  final ValueChanged<String>? onFieldSubmitted;
  final FocusNode? focusNode;
  final TextEditingController? controller;
  final bool autofocus;
  final bool enabled;
  final String labelText;
  final Function(String?) onChanged;

  const PasswordField({
    super.key,
    this.initialValue,
    this.validator,
    this.onFieldSubmitted,
    this.focusNode,
    this.controller,
    this.autofocus = false,
    this.enabled = true,
    required this.labelText,
    required this.onChanged,
  });

  @override
  State<PasswordField> createState() => _PasswordFieldState();
}

class _PasswordFieldState extends State<PasswordField> {
  bool _isHidden = true;

  @override
  Widget build(BuildContext context) {
    var theme = Theme.of(context);
    return TextFormField(
      initialValue: widget.initialValue,
      validator: widget.validator,
      obscureText: _isHidden,
      maxLines: 1,
      keyboardType: TextInputType.text,
      onFieldSubmitted: widget.onFieldSubmitted,
      autofillHints: const [AutofillHints.password],
      focusNode: widget.focusNode,
      controller: widget.controller,
      autofocus: widget.autofocus,
      decoration: InputDecoration(
        enabled: widget.enabled,
        labelText: widget.labelText,
        suffixIcon: InkWell(
          focusNode: FocusNode(skipTraversal: true),
          onTap: _toggleHidden,
          child: Icon(
            Icons.visibility,
            color: _isHidden ? theme.disabledColor : null,
          ),
        ),
      ),
      onChanged: widget.onChanged,
    );
  }

  void _toggleHidden() {
    setState(() => _isHidden = !_isHidden);
  }
}
