import 'package:flutter/material.dart';

class LoadingOverlay extends StatefulWidget {
  const LoadingOverlay({
    super.key,
    this.color,
    this.opacity = 0.5,
    this.progressIndicator = const CircularProgressIndicator(),
    required this.isLoading,
    required this.child,
  });

  final Color? color;
  final double opacity;
  final Widget progressIndicator;
  final bool isLoading;
  final Widget child;

  @override
  State<LoadingOverlay> createState() => _LoadingOverlayState();
}

class _LoadingOverlayState extends State<LoadingOverlay>
    with SingleTickerProviderStateMixin {
  _LoadingOverlayState();
  late AnimationController? _controller;
  late Animation<double>? _animation;
  bool? _overlayVisible;

  @override
  void initState() {
    super.initState();
    _overlayVisible = false;
    _controller = AnimationController(
        vsync: this, duration: const Duration(milliseconds: 300));
    _animation = Tween(begin: 0.0, end: 1.0).animate(_controller!);
    _animation!.addStatusListener((status) {
      if (status == AnimationStatus.forward && mounted) {
        setState(() => {_overlayVisible = true});
      }
      if (status == AnimationStatus.dismissed && mounted) {
        setState(() => {_overlayVisible = false});
      }
    });
    if (widget.isLoading) {
      _controller!.forward();
    }
  }

  @override
  void didUpdateWidget(LoadingOverlay oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!oldWidget.isLoading && widget.isLoading) {
      _controller!.forward();
    }
    if (oldWidget.isLoading && !widget.isLoading) {
      _controller!.reverse();
    }
  }

  @override
  void dispose() {
    _controller!.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final widgets = <Widget>[];
    widgets.add(widget.child);
    if (_overlayVisible ?? false) {
      final modal = Positioned.fill(
        child: FadeTransition(
          opacity: _animation!,
          child: Stack(
            children: <Widget>[
              Opacity(
                opacity: widget.opacity,
                child: ModalBarrier(
                  dismissible: false,
                  color:
                      widget.color ?? Theme.of(context).colorScheme.background,
                ),
              ),
              Center(child: widget.progressIndicator),
            ],
          ),
        ),
      );
      widgets.add(modal);
    }
    return Stack(children: widgets);
  }
}
