import 'package:flutter/material.dart';

import 'item.dart';
import 'route_path.dart';

typedef RouteSegments = Map<String, String?>;

typedef RouteParameters = Map<String, String>;

class ParsingException implements Exception {
  ParsingException();
}

class RouteArguments {
  RouteArguments();

  factory RouteArguments.parse(
    RouteSegments segments,
    RouteParameters params,
  ) {
    return RouteArguments();
  }
}

class RouteBuilder<T extends RouteArguments> {
  final RegExp pattern;
  final String Function(T, String?) path;
  final T Function(RouteSegments, RouteParameters) parser;
  final List<RouteBuilder> paths;
  final Future<Widget> Function(BuildContext, T, RoutePath?) build;
  final Future<RoutePath?> Function(BuildContext, T) rewrite;
  final List<NavigationItem> Function()? menuItems;
  final List<NavigationItem> Function()? tabItems;

  RouteBuilder({
    required this.pattern,
    required this.path,
    required this.parser,
    required this.build,
    required this.rewrite,
    this.paths = const [],
    this.menuItems,
    this.tabItems,
  });

  RoutePath<T> call([RoutePath? parent, RouteArguments? args]) =>
      RoutePath<T>(this, (args ?? RouteArguments()) as T, parent);

  RoutePath<RouteArguments>? parse(
    String path,
    RouteParameters params,
    RoutePath? parent,
  ) {
    var match = pattern.firstMatch(path);
    if (match == null) return null;
    var subPath = match.group(match.groupCount);
    // Collect the segments for the path
    RouteSegments segments = {};
    var groupNames = match.groupNames.map((o) => [o, match.namedGroup(o)]);
    segments = {for (var o in groupNames) '${o[0]}': o[1]};
    var args = parser(segments, params);
    var current = this(parent, args);
    // Check for mathcing sub paths
    if (subPath != null) {
      for (var o in paths) {
        var result = o.parse(subPath, params, current);
        if (result != null) return result;
      }
    }
    return current;
  }
}
