import 'dart:core';

import 'package:flutter/widgets.dart';

import 'item.dart';
import 'route_builder.dart';

class RoutePath<T extends RouteArguments> {
  final RouteBuilder<T> routeBuilder;
  final T args;
  final RoutePath? parent;

  RoutePath(
    this.routeBuilder,
    this.args,
    this.parent,
  );

  RegExp get pattern => routeBuilder.pattern;

  String path([String? subPath]) {
    var parent = this.parent;
    var result = routeBuilder.path(args, subPath ?? '');
    if (parent != null) {
      result = parent.path(result);
    }
    return result;
  }

  List<NavigationItem>? get menuItems {
    var menuItems = routeBuilder.menuItems;
    return menuItems == null ? null : menuItems();
  }

  List<NavigationItem>? get tabItems {
    var tabItems = routeBuilder.tabItems;
    return tabItems == null ? null : tabItems();
  }

  Future<Widget> build(BuildContext context) {
    return routeBuilder.build(context, args, parent);
  }

  Future<RoutePath?> rewrite(BuildContext context) {
    return routeBuilder.rewrite(context, args);
  }

  @override
  bool operator ==(Object other) {
    return other is RoutePath && other.path() == path();
  }

  @override
  int get hashCode => path().hashCode;
}
