import 'package:flutter/material.dart';

import 'logic.dart';
import 'route_path.dart';

class StandardRouterDelegate<User> extends RouterDelegate<RoutePath>
    with ChangeNotifier, PopNavigatorRouterDelegateMixin<RoutePath> {
  final BuildContext context;

  final NavigationLogic _logic;

  @override
  final navigatorKey = GlobalKey<NavigatorState>();

  @override
  get currentConfiguration => _logic.routePath;

  StandardRouterDelegate(this.context, this._logic) {
    _logic.addListener(notifyListeners);
  }

  @override
  Future<bool> popRoute() async {
    return _logic.popRoute();
  }

  @override
  Widget build(BuildContext context) {
    return Navigator(
      key: navigatorKey,
      onPopPage: (route, result) {
        return route.didPop(result);
      },
      pages: [
        MaterialPage(
          child: _logic.currentScreen,
        ),
      ],
    );
  }

  @override
  Future<void> setNewRoutePath(RoutePath configuration) async {
    return _logic.setRoutePath(configuration);
  }
}
